/////////////////////////////////////////////////////////////////////////////////////
// File:	filelock.h
// Author:	Walt Jones
// Email:	wjones@blarg.net
// Date:	29 July, 2000
// Purpose:	Creates a .lock file associated with a given file.  If an attempt is made by
//			another instance of fileLock to place a lock on the same .lock file, the other
//			instance will wait until the orioginal owner of the lock removes it or
//			5000 miliseconds pass.
/////////////////////////////////////////////////////////////////////////////////////


// Make sure this file is included only once
#if !defined( FILELOCK_H )
#define FILELOCK_H

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string>
#include <vector>
#include <time.h>

class fileLock
{
private:
	// Variables
	ifstream fin;			// Input filestream
	ofstream fout;			// Outfile filestream
	char *filename;			// Filename of the lock file
	int timeout;			// Timeout for old locks
	time_t startTime;		// Start time used during timeout computation
	bool locked;			// Denotes if the lock is currently in place

public:
	// Methods
	fileLock();
	fileLock(const string &file) { setFilename(file); fileLock(); return;};
	~fileLock();
	void setFilename(const string &file);
	void lock();
	void release();
	bool isLocked() { return locked;};	// Returns true if the file is currently locked
};

/////////////////////////////////////////////////////////////////////////////////////
// Method:	fileLock
// Purpose:	Constructor
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
fileLock::fileLock()
{
	timeout = 5;	// Timeout for old locks is 5 seconds
	locked = false;
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	~fileLock
// Purpose:	Destructor
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
fileLock::~fileLock()
{
	// Release the lock
	release();
	delete filename;
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	setFilename
// Purpose:	Sets the filename of the lock
// Input:	file - The filename (with full path) of the file to lock
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void fileLock::setFilename(const string &file)
{
	filename = new char[file.length()];
	strcpy(filename, file.c_str());
	strcat(filename, ".lock");
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	lock
// Purpose:	Places a lock on the file filename
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void fileLock::lock()
{
	if (!isLocked())
	{
		startTime = time(NULL);

		// Test to see if a lock file already exists.  If one already exists, wait until it's gone
		// or we timeout
		fin.open(filename);
		while (!fin.fail() && ((time(NULL) - startTime) <= timeout))
		{
			fin.close();
			fin.open(filename);
		}

		// If the old lock file is still there, remove it
		if (!fin.fail())
		{
			fin.close();
			remove(filename);
		}

		// Create the lock file and keep it open
		fout.open(filename);
		locked = true;
	}

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	release
// Purpose:	Releases the lock on the file filename
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void fileLock::release()
{
	// Close files
	fin.close();
	fout.close();

	// If we still have a lock, release it
	if (isLocked())
	{
		remove(filename);
		locked = false;
	}
	return;
}

#endif // defined( FILELOCK_H )
